﻿using System;
using System.Text;
using System.Threading;
using System.Net.Sockets;

// Built with Visual Studio 2012
// Chamber Controller settings:
// Terminator:  Last Input
// Send Acknowledgement: Checked (yes)
// Always receive after a send...even if not needing a result...returned value is chamber controller status

namespace CSharp_socket_non_visa
{
  class ChamberControllerIoUsingTcpSocketWithoutVisa
  {
    static void Main(string[] args)
    {
      //Create new TCP Socket
      Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

      //Connect to Chamber Controller at IP address 192.168.4.97, TCP port 8888.  Replace with your own settings
      // Put chamber Setup Communications Network settings here:
      s.Connect("192.168.4.97", 8888);

      //Use the IDEN? command to retreive chamber idenfication
      //Note that we use carriage return as the termination character for communication
      byte[] command = ASCIIEncoding.ASCII.GetBytes("IDEN?\r");
      //Send the command to the chamber
      s.Send(command);
      //Create new buffer to hold response from Chamber Controller
      byte[] received = new byte[128];
      //Receive response from the Chamber Controller
      int len = s.Receive(received);
      //Convert received ASCII bytes to string
      string temp = ASCIIEncoding.ASCII.GetString(received, 0, len);
      //Print the chamber identification to console
      Console.WriteLine("Chamber Controller Identified as: " + temp);

      RunManual(s); // run the chamber controller in manual mode

      //Keep checking the Chamber for temperature every second, 10 times
      for (float i = 0; i < 10; i++)
      {
        SetSetpointChan1(s, (25F + i)); // Set the temperature setpoint

        //PVAR1? requests the Process Variable for Channel 1
        //Refer to Thermotron Chamber Controller Manual for rest of I/O commands for your model
        command = ASCIIEncoding.ASCII.GetBytes("PVAR1?\r");
        //Send the Query Command
        s.Send(command);
        //Receive response
        len = s.Receive(received);
        //Convert received ASCII bytes to string
        temp = ASCIIEncoding.ASCII.GetString(received, 0, len);
        //Print the chamber temperature to Console
        Console.WriteLine("Count=" + i + " Current Chamber Temperature: " + temp);
        string strResult = ""; ;
        GetSetpointChan1(s, out strResult);
        //Print the chamber setpoint for channel 1 (temperature) to Console
        Console.WriteLine("Current Chamber Temperature Setpoint: " + strResult);
        //Get new value every 1000ms (1 sec)
        Thread.Sleep(1000);
      }

      Stop(s); // Stop the chamber controller
      Console.WriteLine("Press a key to continue");
      Console.ReadKey();

      //Close socket
      s.Close();
    }

    public static void RunManual(Socket socket)
    {
      // Always receive after a send...even if not needing a result...in that case, result returned is chamber controller status
      byte[] byteCommand = ASCIIEncoding.ASCII.GetBytes("RUNM\r");
      //Send the command to the chamber
      socket.Send(byteCommand);
      //Create new buffer to hold response from Chamber Controller
      byte[] byteReceived = new byte[128];
      //Receive response from the Chamber Controller
      int len = socket.Receive(byteReceived);
      Console.WriteLine("Chamber in Run Manual mode");
    }

    public static void SetSetpointChan1(Socket socket, float fSetpointValue)
    {
      byte[] byteCommand = ASCIIEncoding.ASCII.GetBytes("SETP1," + fSetpointValue.ToString() + "\r");
      //Send the command to the chamber
      socket.Send(byteCommand);
      //Create new buffer to hold response from Chamber Controller
      byte[] byteReceived = new byte[128];
      //Receive response from the Chamber Controller
      int len = socket.Receive(byteReceived);
    }

    public static void GetSetpointChan1(Socket socket, out string strValue)
    {
      byte[] byteCommand = ASCIIEncoding.ASCII.GetBytes("SETP1?\r");
      //Send the Query Command
      socket.Send(byteCommand);
      byte[] byteReceived = new byte[128];
      //Receive response
      int len = socket.Receive(byteReceived);
      //Convert received ASCII bytes to string
      strValue = ASCIIEncoding.ASCII.GetString(byteReceived, 0, len);
    }

    public static void Stop(Socket socket)
    {
      // Always receive after a send...even if not needing a result...in that case, result returned is chamber controller status
      byte[] byteCommand = ASCIIEncoding.ASCII.GetBytes("STOP\r");
      //Send the command to the chamber
      socket.Send(byteCommand);
      //Create new buffer to hold response from Chamber Controller
      byte[] byteReceived = new byte[128];
      //Receive response from the Chamber Controller
      int len = socket.Receive(byteReceived);
      Console.WriteLine("Chamber stopped");
    }
  }
}
